#!/usr/bin/tclsh

# Google Translate service script
# Jim DeVona
#
# Outstanding issues:
#	1. displaying an AppleScript UI from a filter ThisService is gross;
#	   the workaround I use here does the job, but steals window focus
#	2. the translation output always includes a newline, which looks
#	   stupid in many situations.
#	3. doesn't load too fast
#
# Requires Mac OS X 10.4. GetLangPair uses osascript to present a UI
# via Applescript. With a different GetLangPair implementation, should
# work on any system with tdom installed (http is distributed with Tcl).
#

package require http
package require tdom

#
# GetLangPair
#
#	Ask the user to select a translation.
#
# Parameters:
#
#	none
#
# Results:
#
#	returns translation langpair chosen by user
#	just exits on error
#
proc GetLangPair {} {

	# we use osascript to display the dialog
	if {![file executable /usr/bin/osascript]} {
		puts stderr "Cannot find executable /usr/bin/osascript"
		exit 1
	}
	
	# translation names and codes per http://translate.google.com/
	# these could be retrieved dynamically, too - but that'd be too slow
	array set translations {
			"Arabic to English"                   "ar|en"
			"Chinese to English"                  "zh|en"
			"Chinese (Simplified to Traditional)" "zh-CN|zh-TW"
			"Chinese (Traditional to Simplified)" "zh-TW|zh-CN"
			"English to Arabic"                   "en|ar"
			"English to Chinese (Simplified)"     "en|zh-CN"
			"English to Chinese (Traditional)"    "en|zh-TW"
			"English to French"                   "en|fr"
			"English to German"                   "en|de"
			"English to Italian"                  "en|it"
			"English to Japanese"                 "en|ja"
			"English to Korean"                   "en|ko"
			"English to Portuguese"               "en|pt"
			"English to Russian"                  "en|ru"
			"English to Spanish"                  "en|es"
			"French to English"                   "fr|en"
			"French to German"                    "fr|de"
			"German to English"                   "de|en"
			"German to French"                    "de|fr"
			"Italian to English"                  "it|en"
			"Japanese to English"                 "ja|en"
			"Korean to English"                   "ko|en"
			"Portuguese to English"               "pt|en"
			"Russian to English"                  "ru|en"
			"Spanish to English"                  "es|en"
	}
	
	# get sorted list of translation names formatted like an AppleScript list
	set names [format {{"%s"}} [join [lsort [array names translations]] {","}]]
	
	# hack per http://anoved.net/2007/09/user-interfaces-for-thisservice-scripts.html
	set script "tell application \"System Events\"
				set _app to name of first process whose frontmost is true
				activate
				return item 1 of (choose from list $names with title \"Google Translate Service\" with prompt \"Select a translation:\" OK button name \"Translate\" without multiple selections allowed and empty selection allowed)
			end tell"


	# display the dialog; have the user select a translation name
	if {[catch {exec /usr/bin/osascript << $script} response]} {
		puts stderr "AppleScript error: $response"
		exit 1
	}
	
	# return the langpair code corresponding to the selected translation name
	return $translations($response)
}

# Google acts all sassy with anonymous agents
::http::config -useragent "Mozilla/5.0"

# assemble a properly formatted query
set query [::http::formatQuery langpair [GetLangPair] text [read stdin] ie UTF8 hl en]

# post the query and retrieve Google's response
if {[catch {::http::geturl "http://translate.google.com/translate_t" -query $query} token]} {
	puts stderr "HTTP error: $token"
	exit 1
}

# parse the results and clean up after the query
if {[catch {
	::dom parse -html [::http::data $token] doc
	$doc documentElement tree
} err]} {
	puts stderr "Parse error: $err"
	exit 1
}
::http::cleanup $token

# Print each line of the translation
# Note: The whitespace that comes out of the translator does not necessarily
#       match the whitespace that went in to the translator. (Here's looking
#       at you, final newlines-or-lack-thereof.) My decision: your problem.
foreach line [$tree selectNodes {descendant::div[@id='result_box']/text()}] {
	puts stdout [$line data]
}

exit 0
